# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.plugins.pigment.pigment_controller import PigmentController
from elisa.plugins.pigment.widgets.list_carrousel import ListCarrousel
from elisa.plugins.pigment.widgets.widget import Widget
from elisa.plugins.pigment.widgets import notifying_list
from elisa.plugins.pigment.graph.image import Image

from elisa.core.input_event import *

import pgm
from pgm.utils.image import cairo_gradient
import math

class SectionsMenuWidget(ListCarrousel):
    pass

class SectionItemWidget(Widget):
    def __init__(self):
        super(SectionItemWidget, self).__init__()

        self.image = Image()
        self.add(self.image)
        self.image.bg_a = 0
        self.image.layout = pgm.IMAGE_SCALED
        self.image.alignment = pgm.IMAGE_BOTTOM
        self.image.x, self.image.y = (0.0, 0.0)
        self.image.width, self.image.height = (1.0, 0.49)
        self.image.visible = True

        # reflection
        self.reflection = Image()
        self.add(self.reflection)
        flip_matrix = pgm.mat4x4_new_predefined(pgm.MAT4X4_FLIP_VERTICAL)
        self.reflection.mapping_matrix = flip_matrix
        self.reflection.bg_a = 0
        self.reflection.layout = pgm.IMAGE_SCALED
        self.reflection.alignment = pgm.IMAGE_TOP
        self.reflection.x, self.reflection.y = (0.0, 0.5)
        self.reflection.width, self.reflection.height = (1.0, 0.5)
        self.reflection.opacity = 120

class SectionsMenuController(PigmentController):
    def set_frontend(self, frontend):
        super(SectionsMenuController, self).set_frontend(frontend)

        self.sensitive = False
        # menu widget setup
        self.menu = SectionsMenuWidget(SectionItemWidget)
        self.widget.add(self.menu)
        self.menu.width, self.menu.height = (1.0, 1.0)
        self.menu.x, self.menu.y = (0.0, -0.05)
        self.menu.visible = True

        self.model = notifying_list.List()

        self.menu.set_model(self.model)
        self.menu.set_renderer(self._renderer)
        self.menu.connect('item-clicked', self._section_clicked_handler)

    def _show_reflection(self, result, widget):
        widget.reflection.visible = True

    def _renderer(self, item, widget):
        # FIXME: not updated automatically on theme change
        theme = self.frontend.get_theme()
        icon_path = theme.get_resource(item.icon)
        cairo_gradient(icon_path, widget.reflection, 0.4)
        dfr = self.frontend.load_from_theme(item.icon, widget.image)
        dfr.addCallback(self._show_reflection, widget)
        return dfr

    def _section_clicked_handler(self, menu, section):
        # only react if we are sensitive
        if not self.sensitive:
            return

        selected_section = self.model[self.menu.selected_item_index]
        if section == selected_section:
            path = '/poblesec/browser'
            controllers = self.frontend.retrieve_controllers(path)
            # we assume there was only one controller created with that path
            browser = controllers[0]
            path = section.controller_path
            label = section.label
            dfr = browser.history.append_controller(path, label)

    def handle_input(self, manager, input_event):
        if self.widget.focus == True:
            if input_event.value == EventValue.KEY_GO_LEFT:
                self.menu.selected_item_index -= 1
                return True
            elif input_event.value == EventValue.KEY_GO_RIGHT:
                self.menu.selected_item_index += 1
                return True
            elif input_event.value == EventValue.KEY_OK:
                try:
                    item = self.menu.model[self.menu.selected_item_index]
                    self._section_clicked_handler(self.menu, item)
                except IndexError:
                    self.debug("selected-item-index out of range: %s" % self.menu.selected_item_index)
                return True
        else:
            return super(SectionsMenuController, self).handle_input(manager, input_event)
